@@ -21,6 +21,7 @@ use warnings FATAL => 'all';
 use Exporter ();
 use Config;
 use Apache::TestConfig ();
+use Test qw/ok skip/;
 
 BEGIN {
     # Apache::Test loads a bunch of mp2 stuff while getting itself
@@ -36,74 +37,96 @@ BEGIN {
 
 use vars qw(@ISA @EXPORT %EXPORT_TAGS $VERSION %SubTests @SkipReasons);
 
-$VERSION = '1.31';
+$VERSION = '1.42';
 
 my @need = qw(need_lwp need_http11 need_cgi need_access need_auth
-              need_module need_apache need_min_apache_version
+              need_module need_apache need_min_apache_version need_min_apache_fix
               need_apache_version need_perl need_min_perl_version
-              need_min_module_version need_threads need_apache_mpm
-              need_php need_php4 need_ssl need_imagemap);
+              need_min_module_version need_threads need_fork need_apache_mpm
+              need_php need_php4 need_ssl need_imagemap need_cache_disk);
 
 my @have = map { (my $need = $_) =~ s/need/have/; $need } @need;
 
 @ISA = qw(Exporter);
-@EXPORT = (qw(ok skip sok plan skip_reason under_construction need),
+@EXPORT = (qw(sok plan skip_reason under_construction need),
            @need, @have);
 
-# everything but ok(), skip(), and plan() - Test::More provides these
-my @test_more_exports = grep { ! /^(ok|skip|plan)$/ } @EXPORT;
-
-%EXPORT_TAGS = (withtestmore => \@test_more_exports);
-
 %SubTests = ();
 @SkipReasons = ();
 
-if (my $subtests = $ENV{HTTPD_TEST_SUBTESTS}) {
-    %SubTests = map { $_, 1 } split /\s+/, $subtests;
+sub cp {
+    my @l;
+    for( my $i=1; (@l=caller $i)[0] eq __PACKAGE__; $i++ ) {};
+    return wantarray ? @l : $l[0];
 }
 
 my $Config;
-my $real_plan;
-my @testmore;
-
+my %wtm;
 sub import {
-    my $class = shift;
+    my $class=$_[0];
+    my $wtm=0;
+    my @base_exp;
+    my @exp;
+    my %my_exports;
+    undef @my_exports{@EXPORT};
 
-    # once Test::More always Test::More until plan() is called
-    if (($_[0] and $_[0] =~ m/^-withtestmore/) || @testmore) {
-        # special hoops for Test::More support
+    my ($caller,$f,$l)=cp;
 
-        $real_plan = eval {
-
-            require Test::More;
-
-            no warnings qw(numeric);
-            Test::Builder->VERSION('0.18_01');
-
-            # required for Test::More::import() and Apache::Test::plan()
-            # if we don't do this, Test::More exports plan() anyway
-            # and we get collisions.  go figure.
-            @testmore = (import => [qw(!plan)]);
-
-            Test::More->import(@testmore);
-
-            \&Test::More::plan;
-        } or die "-withtestmore error: $@";
-
-        # clean up arguments to export_to_level
-        shift;
-        @EXPORT = (@test_more_exports, @Test::More::EXPORT);
+    for( my $i=1; $i<@_; $i++ ) {
+	if( $_[$i] eq '-withtestmore' ) {
+	    $wtm=1;
+	}
+	elsif( $_[$i] eq ':DEFAULT' ) {
+	    push @exp, $_[$i];
+	    push @base_exp, $_[$i];
+	}
+	elsif( $_[$i] eq '!:DEFAULT' ) {
+	    push @exp, $_[$i];
+	    push @base_exp, $_[$i];
+	}
+	elsif( $_[$i]=~m@^[:/!]@ ) {
+	    warn("Ignoring import spec $_[$i] ".
+		 "at $f line $l\n")
+	}
+	elsif( exists $my_exports{$_[$i]} ) {
+	    push @exp, $_[$i];
+	}
+	else {
+	    push @base_exp, $_[$i];
+	}
     }
-    else {
-        # the default - Test.pm support
-
-        require Test;
-        Test->import(qw(ok skip));
-        @testmore = ();               # reset, just in case.
-        $real_plan = \&Test::plan;
+    if (!@exp and @base_exp) {
+	@exp=('!:DEFAULT');
+    }
+    elsif (@exp and !@base_exp) {
+	@base_exp=('!:DEFAULT');
     }
 
-    $class->export_to_level(1, undef, @_ ? @_ : @EXPORT);
+    $wtm{$caller}=[$wtm,$f,$l] unless exists $wtm{$caller};
+
+    warn("Ignoring -withtestmore due to a previous call ".
+	 "($wtm{$caller}->[1]:$wtm{$caller}->[2]) without it ".
+	 "at $f line $l\n")
+	if $wtm{$caller}->[0]==0 and $wtm==1;
+
+    $class->export_to_level(1, $class, @exp);
+
+    push @base_exp, '!plan';
+    if( $wtm{$caller}->[0] ) {	# -withtestmore
+	eval <<"EVAL"
+package $caller;
+#line $l $f
+use Test::More import=>\\\@base_exp;
+EVAL
+    }
+    else {			# -withouttestmore
+	eval <<"EVAL";
+package $caller;
+#line $l $f
+use Test \@base_exp;
+EVAL
+    }
+    die $@ if $@;
 }
 
 sub config {
@@ -125,26 +148,45 @@ sub sok (&;$) {
     my $sub = shift;
     my $nok = shift || 1; #allow sok to have 'ok' within
 
-    if (%SubTests and not $SubTests{ $Test::ntest }) {
-        for my $n (1..$nok) {
-            skip("skipping this subtest", 0);
-        }
-        return;
-    }
+    my ($caller,$f,$l)=cp;
 
-    my($package, $filename, $line) = caller;
+    if (exists $wtm{$caller} and $wtm{$caller}->[0]==1) { # -withtestmore
+	require Test::Builder;
+	my $tb=Test::Builder->new;
 
-    # trick ok() into reporting the caller filename/line when a
-    # sub-test fails in sok()
-    return eval <<EOE;
-#line $line $filename
-    ok(\$sub->());
+	if (%SubTests and not $SubTests{ 1+$tb->current_test }) {
+	    $tb->skip("skipping this subtest") for (1..$nok);
+	    return;
+	}
+
+	# trick ok() into reporting the caller filename/line when a
+	# sub-test fails in sok()
+	return eval <<EOE;
+#line $l $f
+    Test::More::ok(\$sub->());
 EOE
+    }
+    else {
+	if (%SubTests and not $SubTests{ $Test::ntest }) {
+	    skip("skipping this subtest", 0) for (1..$nok);
+	    return;
+	}
+
+	# trick ok() into reporting the caller filename/line when a
+	# sub-test fails in sok()
+	return eval <<EOE;
+#line $l $f
+    Test::ok(\$sub->());
+EOE
+    }
 }
 
 #so Perl's Test.pm can be run inside mod_perl
 sub test_pm_refresh {
-    if (@testmore) {
+    my ($caller,$f,$l)=cp;
+
+    if (exists $wtm{$caller} and $wtm{$caller}->[0]==1) { # -withtestmore
+	require Test::Builder;
         my $builder = Test::Builder->new;
 
         $builder->reset;
@@ -159,11 +201,23 @@ sub test_pm_refresh {
         # later user reports.
         $builder->failure_output(\*STDOUT);
     }
-    else {
-        $Test::TESTOUT = \*STDOUT;
-        $Test::planned = 0;
-        $Test::ntest = 1;
-        %Test::todo = ();
+    else {                                                # -withouttestmore
+	unless (exists $wtm{$caller}) {
+	    warn "You forgot to 'use Apache::Test' in package $caller\n";
+	    $wtm{$caller}=[0,$f,$l];
+	}
+	if (defined &Test::_reset_globals) {
+	    Test::_reset_globals();
+	    # Test.pm uses $TESTOUT=*STDOUT{IO}. We cannot do that
+	    # due to the way SetupEnv works.
+	    $Test::TESTOUT = \*STDOUT;
+	}
+	else {
+	    $Test::TESTOUT = \*STDOUT;
+	    $Test::planned = 0;
+	    $Test::ntest = 1;
+	    %Test::todo = ();
+	}
     }
 }
 
@@ -186,34 +240,6 @@ sub init_test_pm {
     $r->content_type('text/plain');
 }
 
-sub need_http11 {
-    require Apache::TestRequest;
-    if (Apache::TestRequest::install_http11()) {
-        return 1;
-    }
-    else {
-        push @SkipReasons,
-           "LWP version 5.60+ required for HTTP/1.1 support";
-        return 0;
-    }
-}
-
-sub need_ssl {
-    my $vars = vars();
-    need_module([$vars->{ssl_module_name}, 'Net::SSL']);
-}
-
-sub need_lwp {
-    require Apache::TestRequest;
-    if (Apache::TestRequest::has_lwp()) {
-        return 1;
-    }
-    else {
-        push @SkipReasons, "libwww-perl is not installed";
-        return 0;
-    }
-}
-
 sub plan {
     init_test_pm(shift) if ref $_[0];
     test_pm_refresh();
@@ -254,12 +280,56 @@ sub plan {
     }
     @SkipReasons = (); # reset
 
-    $real_plan->(@_, @testmore);
+    my ($caller,$f,$l)=cp;
+
+    %SubTests=();
+    if (my $subtests=$ENV{HTTPD_TEST_SUBTESTS}) {
+	%SubTests=map { $_, 1 } split /\s+/, $subtests;
+    }
+
+    if (exists $wtm{$caller} and $wtm{$caller}->[0]==1) { # -withtestmore
+	Test::More::plan(@_);
+    }
+    else {                                                # -withouttestmore
+	unless (exists $wtm{$caller}) {
+	    warn "You forgot to 'use Apache::Test' in package $caller\n";
+	    $wtm{$caller}=[0,$f,$l];
+	}
+	Test::plan(@_);
+    }
 
     # add to Test.pm verbose output
     print "# Using Apache/Test.pm version $VERSION\n";
 }
 
+sub need_http11 {
+    require Apache::TestRequest;
+    if (Apache::TestRequest::install_http11()) {
+        return 1;
+    }
+    else {
+        push @SkipReasons,
+           "LWP version 5.60+ required for HTTP/1.1 support";
+        return 0;
+    }
+}
+
+sub need_ssl {
+    my $vars = vars();
+    need_module([$vars->{ssl_module_name}, 'IO::Socket::SSL']);
+}
+
+sub need_lwp {
+    require Apache::TestRequest;
+    if (Apache::TestRequest::has_lwp()) {
+        return 1;
+    }
+    else {
+        push @SkipReasons, "libwww-perl is not installed";
+        return 0;
+    }
+}
+
 sub need {
     my $need_all = 1;
     for my $cond (@_) {
@@ -348,9 +418,14 @@ sub need_min_module_version {
 }
 
 sub need_cgi {
-    return _need_multi(qw(cgi cgid));
+    return _need_multi(qw(cgi.c cgid.c));
 }
 
+sub need_cache_disk {
+    return _need_multi(qw(cache_disk.c disk_cache.c));
+}
+
+
 sub need_php {
     return _need_multi(qw(php4 php5 sapi_apache2.c));
 }
@@ -424,6 +499,37 @@ sub need_min_apache_version {
     }
 }
 
+sub need_min_apache_fix {
+    my @wantlevels = @_;
+    my $cfg = Apache::Test::config();
+    (my $current) = $cfg->{server}->{version} =~ m:^Apache/((\d)\.(\d+)\.(\d+)):;
+    my $current_major = $2;
+    my $current_minor = $3;
+    my $current_micro = $4;
+
+    foreach(@wantlevels) { 
+        if ($_ =~ m/(\d)\.(\d+)\.(\d+)/) { 
+            my $wanted_major = $1;
+            my $wanted_minor = $2;
+            my $wanted_micro = $3;
+            if ($wanted_major eq $current_major && $wanted_minor eq $current_minor) { 
+                if ($wanted_micro > $current_micro) {
+                    push @SkipReasons,
+                         "apache version $_ or higher is required," .
+                             " this is version $current";
+                    return 0;
+                }
+                else { 
+                    return 1;
+                }
+            }
+        }
+    }
+
+    # We didn't match major+minor, run the test and let the author sort it out
+    return 1;
+}
+
 sub need_apache_version {
     my $wanted = shift;
     my $cfg = Apache::Test::config();
@@ -519,6 +625,22 @@ sub need_threads {
     return $status;
 }
 
+sub need_fork {
+    my $have_fork = $Config{d_fork} ||
+                    $Config{d_pseudofork} ||
+                    (($^O eq 'MSWin32' || $^O eq 'NetWare') &&
+                     $Config{useithreads} &&
+                     $Config{ccflags} =~ /-DPERL_IMPLICIT_SYS/);
+
+     if (!$have_fork) {
+         push @SkipReasons, 'The fork function is unimplemented';
+         return 0;
+     }
+     else {
+         return 1;
+     }
+}
+
 sub under_construction {
     push @SkipReasons, "This test is under construction";
     return 0;
@@ -556,6 +678,8 @@ for my $func (@have) {
 
 package Apache::TestToString;
 
+Apache::Test->import('!:DEFAULT');
+
 sub TIEHANDLE {
     my $string = "";
     bless \$string;
@@ -732,6 +856,13 @@ Require LWP support.
 
 Requires mod_cgi or mod_cgid to be installed.
 
+=item need_cache_disk
+
+  plan tests => 5, need_cache_disk
+
+Requires mod_cache_disk or mod_disk_cache to be installed.
+
+
 =item need_php
 
   plan tests => 5, need_php;
@@ -782,6 +913,16 @@ For example:
 
 requires Apache 2.0.40.
 
+=item need_min_apache_fix
+
+Used to require a particular micro version from corresponding minor release
+
+For example:
+
+  plan tests => 5, need_min_apache_fix("2.0.40", "2.2.30", "2.4.18");
+
+requires Apache 2.0.40 or higher.
+
 =item need_apache_mpm
 
 Used to require a specific Apache Multi-Processing Module.
@@ -815,6 +956,10 @@ For example:
 
 requires Perl 5.8.1 or higher.
 
+=item need_fork
+
+Requires the perl built-in function C<fork> to be implemented.
+
 =item need_module
 
   plan tests => 5, need_module 'CGI';
@@ -936,6 +1081,8 @@ if no reason is given a default reason will be used.
 
 =head1 Additional Configuration Variables
 
+=over 4
+
 =item basic_config
 
   my $basic_cfg = Apache::Test::basic_config();
@@ -968,6 +1115,8 @@ If no arguments are passed, the reference to the variables hash is
 returned. If one or more arguments are passed the corresponding values
 are returned.
 
+=back
+
 =head1 Test::More Integration
 
 There are a few caveats if you want to use I<Apache::Test> with
