@@ -24,11 +24,6 @@ void *modperl_config_dir_create(apr_pool_t *p, char *dir)
 
     MP_TRACE_d(MP_FUNC, "dir %s", dir);
 
-#ifdef USE_ITHREADS
-    /* defaults to per-server scope */
-    dcfg->interp_scope = MP_INTERP_SCOPE_UNDEF;
-#endif
-
     return dcfg;
 }
 
@@ -114,10 +109,6 @@ void *modperl_config_dir_merge(apr_pool_t *p, void *basev, void *addv)
                (unsigned long)basev, (unsigned long)addv,
                (unsigned long)mrg);
 
-#ifdef USE_ITHREADS
-    merge_item(interp_scope);
-#endif
-
     mrg->flags = modperl_options_merge(p, base->flags, add->flags);
 
     merge_item(location);
@@ -262,8 +253,6 @@ void *modperl_config_srv_create(apr_pool_t *p, server_rec *s)
         (modperl_tipool_config_t *)
         apr_pcalloc(p, sizeof(*scfg->interp_pool_cfg));
 
-    scfg->interp_scope = MP_INTERP_SCOPE_REQUEST;
-
     /* XXX: determine reasonable defaults */
     scfg->interp_pool_cfg->start = 3;
     scfg->interp_pool_cfg->max_spare = 3;
@@ -308,7 +297,6 @@ void *modperl_config_srv_merge(apr_pool_t *p, void *basev, void *addv)
 
 #ifdef USE_ITHREADS
     merge_item(interp_pool_cfg);
-    merge_item(interp_scope);
 #else
     merge_item(perl);
 #endif
@@ -374,9 +362,14 @@ apr_status_t modperl_config_request_cleanup(pTHX_ request_rec *r)
 apr_status_t modperl_config_req_cleanup(void *data)
 {
     request_rec *r = (request_rec *)data;
-    MP_dTHX;
+    apr_status_t rc;
+    MP_dINTERPa(r, NULL, NULL);
 
-    return modperl_config_request_cleanup(aTHX_ r);
+    rc = modperl_config_request_cleanup(aTHX_ r);
+
+    MP_INTERP_PUTBACK(interp, aTHX);
+
+    return rc;
 }
 
 void *modperl_get_perl_module_config(ap_conf_vector_t *cv)
@@ -674,3 +667,10 @@ int modperl_config_is_perl_option_enabled(pTHX_ request_rec *r,
     }
 
 }
+
+/*
+ * Local Variables:
+ * c-basic-offset: 4
+ * indent-tabs-mode: nil
+ * End:
+ */
