@@ -37,12 +37,13 @@ static unsigned long modperl_interp_address(pTHX)
 #define MP_ENV_HV_STORE(hv, key, val) STMT_START {              \
         I32 klen = strlen(key);                                 \
         SV **svp = hv_fetch(hv, key, klen, FALSE);              \
+        SV *sv;                                                 \
                                                                 \
         if (svp) {                                              \
             sv_setpv(*svp, val);                                \
         }                                                       \
         else {                                                  \
-            SV *sv = newSVpv(val, 0);                           \
+            sv = newSVpv(val, 0);                               \
             (void)hv_store(hv, key, klen, sv, FALSE);           \
             modperl_envelem_tie(sv, key, klen);                 \
             svp = &sv;                                          \
@@ -120,6 +121,7 @@ static void modperl_env_table_populate(pTHX_ apr_table_t *table)
     const apr_array_header_t *array;
     apr_table_entry_t *elts;
 
+    modperl_env_init(aTHX);
     modperl_env_untie(mg_flags);
 
     array = apr_table_elts(table);
@@ -232,15 +234,15 @@ void modperl_env_configure_server(pTHX_ apr_pool_t *p, server_rec *s)
         }
     }
 
-    MP_TRACE_e(MP_FUNC, "\t[%s/0x%lx/%s]"
+    MP_TRACE_e(MP_FUNC, "\t[0x%lx/%s]"
                "\n\t@ENV{keys scfg->SetEnv} = values scfg->SetEnv;",
-               modperl_pid_tid(p), modperl_interp_address(aTHX),
+               modperl_interp_address(aTHX),
                modperl_server_desc(s, p));
     modperl_env_table_populate(aTHX_ scfg->SetEnv);
 
-    MP_TRACE_e(MP_FUNC, "\t[%s/0x%lx/%s]"
+    MP_TRACE_e(MP_FUNC, "\t[0x%lx/%s]"
                "\n\t@ENV{keys scfg->PassEnv} = values scfg->PassEnv;",
-               modperl_pid_tid(p), modperl_interp_address(aTHX),
+               modperl_interp_address(aTHX),
                modperl_server_desc(s, p));
     modperl_env_table_populate(aTHX_ scfg->PassEnv);
 }
@@ -270,9 +272,9 @@ void modperl_env_configure_request_dir(pTHX_ request_rec *r)
          * collisions with per-server PerlSetEnv entries are
          * resolved via the nature of a Perl hash
          */
-        MP_TRACE_e(MP_FUNC, "\t[%s/0x%lx/%s]"
+        MP_TRACE_e(MP_FUNC, "\t[0x%lx/%s]"
                    "\n\t@ENV{keys dcfg->SetEnv} = values dcfg->SetEnv;",
-                   modperl_pid_tid(r->pool), modperl_interp_address(aTHX),
+                   modperl_interp_address(aTHX),
                    modperl_server_desc(r->server, r->pool));
         modperl_env_table_populate(aTHX_ dcfg->SetEnv);
 
@@ -303,9 +305,9 @@ void modperl_env_configure_request_srv(pTHX_ request_rec *r)
      */
 
     if (!apr_is_empty_table(scfg->SetEnv)) {
-        MP_TRACE_e(MP_FUNC, "\t[%s/0x%lx/%s]"
+        MP_TRACE_e(MP_FUNC, "\t[0x%lx/%s]"
                    "\n\t@ENV{keys scfg->SetEnv} = values scfg->SetEnv;",
-                   modperl_pid_tid(r->pool), modperl_interp_address(aTHX),
+                   modperl_interp_address(aTHX),
                    modperl_server_desc(r->server, r->pool));
         modperl_env_table_populate(aTHX_ scfg->SetEnv);
 
@@ -313,9 +315,9 @@ void modperl_env_configure_request_srv(pTHX_ request_rec *r)
     }
 
     if (!apr_is_empty_table(scfg->PassEnv)) {
-        MP_TRACE_e(MP_FUNC, "\t[%s/0x%lx/%s]"
+        MP_TRACE_e(MP_FUNC, "\t[0x%lx/%s]"
                    "\n\t@ENV{keys scfg->PassEnv} = values scfg->PassEnv;",
-                   modperl_pid_tid(r->pool), modperl_interp_address(aTHX),
+                   modperl_interp_address(aTHX),
                    modperl_server_desc(r->server, r->pool));
         modperl_env_table_populate(aTHX_ scfg->PassEnv);
 
@@ -363,9 +365,9 @@ void modperl_env_request_populate(pTHX_ request_rec *r)
      * phases from populating %ENV with new subprocess_env table entries
      */
 
-    MP_TRACE_e(MP_FUNC, "\t[%s/0x%lx/%s%s]"
+    MP_TRACE_e(MP_FUNC, "\t[0x%lx/%s%s]"
                "\n\t@ENV{keys r->subprocess_env} = values r->subprocess_env;",
-               modperl_pid_tid(r->pool), modperl_interp_address(aTHX),
+               modperl_interp_address(aTHX),
                modperl_server_desc(r->server, r->pool), r->uri);
 
     /* we can eliminate some of the cost by only doing CGI variables once
@@ -397,8 +399,8 @@ void modperl_env_request_unpopulate(pTHX_ request_rec *r)
     }
 
     MP_TRACE_e(MP_FUNC,
-               "\n\t[%s/0x%lx/%s%s]\n\tdelete @ENV{keys r->subprocess_env};",
-               modperl_pid_tid(r->pool), modperl_interp_address(aTHX),
+               "\n\t[0x%lx/%s%s]\n\tdelete @ENV{keys r->subprocess_env};",
+               modperl_interp_address(aTHX),
                modperl_server_desc(r->server, r->pool), r->uri);
     modperl_env_table_unpopulate(aTHX_ r->subprocess_env);
 
@@ -411,8 +413,8 @@ void modperl_env_request_tie(pTHX_ request_rec *r)
     EnvMgLenSet(-1);
 
 #ifdef MP_PERL_HV_GMAGICAL_AWARE
-    MP_TRACE_e(MP_FUNC, "[%s/0x%lx] tie %%ENV, $r\t (%s%s)",
-               modperl_pid_tid(r->pool), modperl_interp_address(aTHX),
+    MP_TRACE_e(MP_FUNC, "[0x%lx] tie %%ENV, $r\t (%s%s)",
+               modperl_interp_address(aTHX),
                modperl_server_desc(r->server, r->pool), r->uri);
     SvGMAGICAL_on((SV*)ENVHV);
 #endif
@@ -423,21 +425,17 @@ void modperl_env_request_untie(pTHX_ request_rec *r)
     EnvMgObjSet(NULL);
 
 #ifdef MP_PERL_HV_GMAGICAL_AWARE
-    MP_TRACE_e(MP_FUNC, "[%s/0x%lx] untie %%ENV; # from r\t (%s%s)",
-               modperl_pid_tid(r->pool), modperl_interp_address(aTHX),
+    MP_TRACE_e(MP_FUNC, "[0x%lx] untie %%ENV; # from r\t (%s%s)",
+               modperl_interp_address(aTHX),
                modperl_server_desc(r->server, r->pool), r->uri);
     SvGMAGICAL_off((SV*)ENVHV);
 #endif
 }
 
-/* to store the original virtual tables
- * these are global, not per-interpreter
+/* handy access to perl's original virtual tables
  */
-static MGVTBL MP_PERL_vtbl_env;
-static MGVTBL MP_PERL_vtbl_envelem;
-
 #define MP_PL_vtbl_call(name, meth) \
-    MP_PERL_vtbl_##name.svt_##meth(aTHX_ sv, mg)
+    PL_vtbl_##name.svt_##meth(aTHX_ sv, mg)
 
 #define MP_dENV_KEY \
     STRLEN klen; \
@@ -480,8 +478,7 @@ static int modperl_env_magic_set_all(pTHX_ SV *sv, MAGIC *mg)
                 apr_table_set(r->subprocess_env,
                               hv_iterkey(entry, &keylen),
                               SvPV(hv_iterval((HV*)sv, entry), n_a));
-                MP_TRACE_e(MP_FUNC, "[%s/0x%lx] localizing: %s => %s",
-                           modperl_pid_tid(r->pool),
+                MP_TRACE_e(MP_FUNC, "[0x%lx] localizing: %s => %s",
                            modperl_interp_address(aTHX),
                            hv_iterkey(entry, &keylen),
                            SvPV(hv_iterval((HV*)sv, entry), n_a));
@@ -493,17 +490,14 @@ static int modperl_env_magic_set_all(pTHX_ SV *sv, MAGIC *mg)
         HE *entry;
         STRLEN n_a;
 
-        MP_TRACE_e(MP_FUNC,
-                   "\n\t[%lu/0x%lx] populating %%ENV:",
-                   (unsigned long)getpid(), modperl_interp_address(aTHX));
+        MP_TRACE_e(MP_FUNC, "\n\t[0x%lx] populating %%ENV:",
+                   modperl_interp_address(aTHX));
 
         hv_iterinit((HV*)sv);
 
         while ((entry = hv_iternext((HV*)sv))) {
                 I32 keylen;
                 MP_TRACE_e(MP_FUNC, "$ENV{%s} = \"%s\";",
-                           modperl_pid_tid(r->pool),
-                           modperl_interp_address(aTHX),
                            hv_iterkey(entry, &keylen),
                            SvPV(hv_iterval((HV*)sv, entry), n_a));
             }
@@ -520,20 +514,38 @@ static int modperl_env_magic_clear_all(pTHX_ SV *sv, MAGIC *mg)
 
     if (r) {
         apr_table_clear(r->subprocess_env);
-        MP_TRACE_e(MP_FUNC,
-                   "[%s/0x%lx] clearing all magic off r->subprocess_env",
-                   modperl_pid_tid(r->pool), modperl_interp_address(aTHX));
+        MP_TRACE_e(MP_FUNC, "[0x%lx] clearing all magic off r->subprocess_env",
+                   modperl_interp_address(aTHX));
     }
     else {
-        MP_TRACE_e(MP_FUNC,
-                   "[%s/0x%lx] %%ENV = ();",
-                   modperl_pid_tid(r->pool), modperl_interp_address(aTHX));
+        MP_TRACE_e(MP_FUNC, "[0x%lx] %%ENV = ();",
+                   modperl_interp_address(aTHX));
         return MP_PL_vtbl_call(env, clear);
     }
 
     return 0;
 }
 
+static int modperl_env_magic_copy(pTHX_ SV *sv, MAGIC *mg, SV *nsv, const char *name, I32 namlen)
+{
+    MP_TRACE_e(MP_FUNC, "setting up %%ENV element magic");
+    sv_magicext(nsv, mg->mg_obj, toLOWER(mg->mg_type), &MP_vtbl_envelem, name, namlen);
+
+    return 1;
+}
+
+static int modperl_env_magic_local_all(pTHX_ SV *nsv, MAGIC *mg)
+{
+    MAGIC *nmg;
+    MP_TRACE_e(MP_FUNC, "localizing %%ENV");
+    nmg = sv_magicext(nsv, mg->mg_obj, mg->mg_type, &MP_vtbl_env, (char*)NULL, 0);
+    nmg->mg_ptr = mg->mg_ptr;
+    nmg->mg_flags |= MGf_COPY;
+    nmg->mg_flags |= MGf_LOCAL;
+
+    return 1;
+}
+
 static int modperl_env_magic_set(pTHX_ SV *sv, MAGIC *mg)
 {
     request_rec *r = (request_rec *)EnvMgObj;
@@ -542,8 +554,7 @@ static int modperl_env_magic_set(pTHX_ SV *sv, MAGIC *mg)
         MP_dENV_KEY;
         MP_dENV_VAL;
         apr_table_set(r->subprocess_env, key, val);
-        MP_TRACE_e(MP_FUNC, "[%s/0x%lx] r->subprocess_env set: %s => %s",
-                   modperl_pid_tid(r->pool),
+        MP_TRACE_e(MP_FUNC, "[0x%lx] r->subprocess_env set: %s => %s",
                    modperl_interp_address(aTHX), key, val);
     }
     else {
@@ -551,8 +562,7 @@ static int modperl_env_magic_set(pTHX_ SV *sv, MAGIC *mg)
         MP_dENV_KEY;
         MP_dENV_VAL;
         MP_TRACE_e(MP_FUNC,
-                   "[%lu/0x%lx] $ENV{%s} = \"%s\";",
-                   (unsigned long)getpid(),
+                   "[0x%lx] $ENV{%s} = \"%s\";",
                    modperl_interp_address(aTHX), key, val);
 #endif
         return MP_PL_vtbl_call(envelem, set);
@@ -568,15 +578,13 @@ static int modperl_env_magic_clear(pTHX_ SV *sv, MAGIC *mg)
     if (r) {
         MP_dENV_KEY;
         apr_table_unset(r->subprocess_env, key);
-        MP_TRACE_e(MP_FUNC, "[%s/0x%lx] r->subprocess_env unset: %s",
-                   modperl_pid_tid(r->pool),
+        MP_TRACE_e(MP_FUNC, "[0x%lx] r->subprocess_env unset: %s",
                    modperl_interp_address(aTHX), key);
     }
     else {
 #ifdef MP_TRACE
         MP_dENV_KEY;
-        MP_TRACE_e(MP_FUNC, "[%lu/0x%lx] delete $ENV{%s};",
-                   (unsigned long)getpid(),
+        MP_TRACE_e(MP_FUNC, "[0x%lx] delete $ENV{%s};",
                    modperl_interp_address(aTHX), key);
 #endif
         return MP_PL_vtbl_call(envelem, clear);
@@ -597,15 +605,13 @@ static int modperl_env_magic_get(pTHX_ SV *sv, MAGIC *mg)
         if ((val = apr_table_get(r->subprocess_env, key))) {
             sv_setpv(sv, val);
             MP_TRACE_e(MP_FUNC,
-                       "[%s/0x%lx] r->subprocess_env get: %s => %s",
-                       modperl_pid_tid(r->pool),
+                       "[0x%lx] r->subprocess_env get: %s => %s",
                        modperl_interp_address(aTHX), key, val);
         }
         else {
             sv_setsv(sv, &PL_sv_undef);
             MP_TRACE_e(MP_FUNC,
-                       "[%s/0x%lx] r->subprocess_env get: %s => undef",
-                       modperl_pid_tid(r->pool),
+                       "[0x%lx] r->subprocess_env get: %s => undef",
                        modperl_interp_address(aTHX), key);
         }
     }
@@ -614,8 +620,7 @@ static int modperl_env_magic_get(pTHX_ SV *sv, MAGIC *mg)
 #ifdef MP_TRACE
         MP_dENV_KEY;
         MP_TRACE_e(MP_FUNC,
-                   "[%lu/0x%lx] there is no svt_get in PL_vtbl_envelem: %s",
-                   (unsigned long)getpid(),
+                   "[0x%lx] there is no svt_get in PL_vtbl_envelem: %s",
                    modperl_interp_address(aTHX), key);
 #endif
     }
@@ -625,15 +630,18 @@ static int modperl_env_magic_get(pTHX_ SV *sv, MAGIC *mg)
 #endif
 
 /* override %ENV virtual tables with our own */
-static MGVTBL MP_vtbl_env = {
+MGVTBL MP_vtbl_env = {
     0,
     modperl_env_magic_set_all,
     0,
     modperl_env_magic_clear_all,
-    0
+    0,
+    modperl_env_magic_copy,
+    0,
+    modperl_env_magic_local_all
 };
 
-static MGVTBL MP_vtbl_envelem = {
+MGVTBL MP_vtbl_envelem = {
     0,
     modperl_env_magic_set,
     0,
@@ -641,20 +649,73 @@ static MGVTBL MP_vtbl_envelem = {
     0
 };
 
-void modperl_env_init(void)
+void modperl_env_init(pTHX)
 {
-    /* save originals */
-    StructCopy(&PL_vtbl_env, &MP_PERL_vtbl_env, MGVTBL);
-    StructCopy(&PL_vtbl_envelem, &MP_PERL_vtbl_envelem, MGVTBL);
+    MAGIC *mg;
 
-    /* replace with our versions */
-    StructCopy(&MP_vtbl_env, &PL_vtbl_env, MGVTBL);
-    StructCopy(&MP_vtbl_envelem, &PL_vtbl_envelem, MGVTBL);
+    /* Find the 'E' magic on %ENV */
+    if (!PL_envgv)
+        return;
+    if (!SvRMAGICAL(ENVHV))
+        return;
+    mg = mg_find((const SV *)ENVHV, PERL_MAGIC_env);
+    if (!mg)
+        return;
+       
+    /* Ignore it if it isn't perl's original version */
+    if (mg->mg_virtual != &PL_vtbl_env)
+        return;
+
+    MP_TRACE_e(MP_FUNC, "env_init - ptr: %x obj: %x flags: %x",
+               mg->mg_ptr, mg->mg_obj, mg->mg_flags);
+
+    /* Remove it */
+#if MP_PERL_VERSION_AT_LEAST(5, 13, 6)
+    mg_free_type((SV*)ENVHV, PERL_MAGIC_env);
+#else
+    mg_free((SV*)ENVHV);
+#endif
+
+    /* Add our version instead */
+    mg = sv_magicext((SV*)ENVHV, (SV*)NULL, PERL_MAGIC_env, &MP_vtbl_env, (char*)NULL, 0);
+    mg->mg_flags |= MGf_COPY;
+    mg->mg_flags |= MGf_LOCAL;
 }
 
-void modperl_env_unload(void)
+void modperl_env_unload(pTHX)
 {
-    /* restore originals */
-    StructCopy(&MP_PERL_vtbl_env, &PL_vtbl_env, MGVTBL);
-    StructCopy(&MP_PERL_vtbl_envelem, &PL_vtbl_envelem, MGVTBL);
+    MAGIC *mg;
+
+    /* Find the 'E' magic on %ENV */
+    if (!PL_envgv)
+        return;
+    if (!SvRMAGICAL(ENVHV))
+        return;
+    mg = mg_find((const SV *)ENVHV, PERL_MAGIC_env);
+    if (!mg)
+        return;
+
+    /* Ignore it if it isn't our version */
+    if (mg->mg_virtual != &MP_vtbl_env)
+        return;
+
+    MP_TRACE_e(MP_FUNC, "env_unload - ptr: %x obj: %x flags: %x",
+               mg->mg_ptr, mg->mg_obj, mg->mg_flags);
+
+    /* Remove it */
+#if MP_PERL_VERSION_AT_LEAST(5, 13, 6)
+    mg_free_type((SV*)ENVHV, PERL_MAGIC_env);
+#else
+    mg_free((SV*)ENVHV);
+#endif
+
+    /* Restore perl's original version */
+    sv_magicext((SV*)ENVHV, (SV*)NULL, PERL_MAGIC_env, &PL_vtbl_env, (char*)NULL, 0);
 }
+
+/*
+ * Local Variables:
+ * c-basic-offset: 4
+ * indent-tabs-mode: nil
+ * End:
+ */
