@@ -181,6 +181,8 @@ void modperl_perl_destruct(PerlInterpreter *perl)
         }
     }
 
+    modperl_env_unload(aTHX);
+
     perl_destruct(perl);
 
     /* XXX: big bug in 5.6.1 fixed in 5.7.2+
@@ -280,3 +282,10 @@ void modperl_hash_seed_set(pTHX)
     }
 #endif
 }
+
+/*
+ * Local Variables:
+ * c-basic-offset: 4
+ * indent-tabs-mode: nil
+ * End:
+ */
