@@ -1,3 +1,4 @@
+# please insert nothing before this line: -*- mode: cperl; cperl-indent-level: 4; cperl-continued-statement-offset: 4; indent-tabs-mode: nil -*-
 package TestAPI::access2;
 
 # testing $r->requires
@@ -44,7 +45,6 @@ my %users = (
 
 sub handler {
     my $r = shift;
-
     die '$r->some_auth_required failed' unless $r->some_auth_required;
 
     my $satisfies = $r->satisfies;
@@ -90,7 +90,10 @@ sub handler {
 
 1;
 __DATA__
+
 <NoAutoConfig>
+<IfModule mod_version.c>
+<IfVersion < 2.3.0>
 <Location /TestAPI__access2>
     PerlAuthenHandler TestAPI::access2
     PerlResponseHandler Apache::TestHandler::ok1
@@ -117,4 +120,6 @@ __DATA__
         AuthGroupFile @DocumentRoot@/api/auth-groups
     </IfModule>
 </Location>
+</IfVersion>
+</IfModule>
 </NoAutoConfig>
