@@ -1,3 +1,4 @@
+# please insert nothing before this line: -*- mode: cperl; cperl-indent-level: 4; cperl-continued-statement-offset: 4; indent-tabs-mode: nil -*-
 package TestAPR::finfo;
 
 use strict;
@@ -14,7 +15,6 @@ use TestAPRlib::finfo;
 use APR::Finfo ();
 
 use Apache2::Const -compile => 'OK';
-use APR::Const    -compile => qw(FINFO_NORM);
 
 sub handler {
     my $r = shift;
