@@ -1,3 +1,4 @@
+# please insert nothing before this line: -*- mode: cperl; cperl-indent-level: 4; cperl-continued-statement-offset: 4; indent-tabs-mode: nil -*-
 package TestModperl::setauth;
 
 use strict;
@@ -15,7 +16,7 @@ sub handler {
 
     plan $r, tests => 2;
 
-    ok t_cmp($r->auth_type(), undef, 'auth_type');
+     ok t_cmp($r->auth_type(), "none", 'auth_type');
 
     t_server_log_error_is_expected();
     $r->get_basic_auth_pw();
